/*Network monitor for system tray*/
/*(c) Copyright Barry Kauler 2010, bkhome.org*/
/*GPL license: /usr/share/doc/legal/gpl-2.0.txt*/
/*forst release, version 2.0, may 18, 2010*/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <glib/gstdio.h>
#include <dirent.h>
#include <errno.h>

#include "networkblank.xpm"
#include "networkboth.xpm"
#include "networkdead.xpm"
#include "networkin.xpm"
#include "networkout.xpm"

GdkPixbuf *networkblank_pixbuf;
GdkPixbuf *networkboth_pixbuf;
GdkPixbuf *networkdead_pixbuf;
GdkPixbuf *networkin_pixbuf;
GdkPixbuf *networkout_pixbuf;

GtkStatusIcon *tray_icon;
unsigned int interval = 500; /*update interval in milliseconds*/

char *netdir="/sys/class/net";
DIR *dip;
struct dirent *dit;
int  i = 0;
char buf1[1024];
FILE *fp;
char char1;
int flagactive=0;
int flagactiveprev=0;
char infomsg[256];
char rxstr[52];
long int rxacc=0;
char rxaccstr[64];
char txstr[52];
long int txacc=0;
char txaccstr[64];
long int rxaccprev=0;
long int txaccprev=0;
int flagtransfer=0;
int flagtransferprev=0;

gboolean Update(gpointer ptr);
int find_active();

int find_active() {

    flagactive=0;
    infomsg[0]=0;
    strcat(infomsg,"Active interfaces: ");
    
    dip=opendir(netdir);
    if (dip==NULL) return 1;
    
    while ((dit = readdir(dip)) != NULL)
    {
        i++;
        if (strcmp(dit->d_name,".")!=0 && strcmp(dit->d_name,"..")!=0 && strcmp(dit->d_name,"lo")!=0 ) {
            buf1[0]=0;
            strcat(buf1,netdir);
            strcat(buf1,"/");
            strcat(buf1,dit->d_name);
            strcat(buf1,"/dormant");
            fp=fopen(buf1,"r");
            if (fp!=NULL) {
                char1=fgetc(fp);
                fclose(fp);
                //if interface active, char1==0...
                if (char1=='0') {
                    flagactive=flagactive+1;
                    strcat(infomsg,dit->d_name);
                    strcat(infomsg," ");
                    //accumulate bytes received...
                    buf1[0]=0;
                    strcat(buf1,netdir);
                    strcat(buf1,"/");
                    strcat(buf1,dit->d_name);
                    strcat(buf1,"/statistics/rx_bytes");
                    fp=fopen(buf1,"r");
                    if (fp!=NULL) {
                        fgets(rxstr,12,fp);
                        rxacc=atol(rxstr)/1024;
                        fclose(fp);
                    }
                    //accumulate bytes transmitted...
                    buf1[0]=0;
                    strcat(buf1,netdir);
                    strcat(buf1,"/");
                    strcat(buf1,dit->d_name);
                    strcat(buf1,"/statistics/tx_bytes");
                    fp=fopen(buf1,"r");
                    if (fp!=NULL) {
                        fgets(txstr,12,fp);
                        txacc=atol(txstr)/1024;
                        fclose(fp);
                    }
                    
                }
            }
        }
    }

    closedir(dip);
    if (flagactive==0) strcat(infomsg,"none ");
    sprintf(rxaccstr,"%.2f",((float)rxacc/1024.0));
    strcat(infomsg," Rx:");
    strcat(infomsg,rxaccstr);
    strcat(infomsg,"MB ");
    sprintf(txaccstr,"%.2f",((float)txacc/1024.0));
    strcat(infomsg," Tx:");
    strcat(infomsg,txaccstr);
    strcat(infomsg,"MB");
}

gboolean Update(gpointer ptr) {

    find_active();
    
    if (flagactive==0) flagtransfer=0;
    else {
        flagtransfer=1;
        if (rxacc!=rxaccprev) flagtransfer=2;
        if (txacc!=txaccprev) flagtransfer=3;
        if (txacc!=txaccprev && rxacc!=rxaccprev) flagtransfer=4;
    }
    //printf("flagtransfer=%i flagactive=%i\n",flagtransfer,flagactive); fflush(stdout); //TEST

    if (flagactive==flagactiveprev && flagtransfer==flagtransferprev && flagtransfer==1) return; //no change.
    if (flagactive==flagactiveprev && flagtransfer==flagtransferprev && flagactive==0) return; //no change.
    flagactiveprev=flagactive;
    txaccprev=txacc;
    rxaccprev=rxacc;
    flagtransferprev=flagtransfer;
    
    if (flagtransfer==0) gtk_status_icon_set_from_pixbuf(tray_icon,networkdead_pixbuf);
    else if (flagtransfer==1) gtk_status_icon_set_from_pixbuf(tray_icon,networkblank_pixbuf);
    else if (flagtransfer==2) gtk_status_icon_set_from_pixbuf(tray_icon,networkin_pixbuf);
    else if (flagtransfer==3) gtk_status_icon_set_from_pixbuf(tray_icon,networkout_pixbuf);
    else if (flagtransfer==4) gtk_status_icon_set_from_pixbuf(tray_icon,networkboth_pixbuf);
    
    //update tooltip...
    gtk_status_icon_set_tooltip(tray_icon, infomsg);
}

void tray_icon_on_click(GtkStatusIcon *status_icon, gpointer user_data)
{
    system("ipinfo & ");
}

void tray_icon_on_menu(GtkStatusIcon *status_icon, guint button, guint activate_time, gpointer user_data)
{
        printf("Popup menu\n");
}

static GtkStatusIcon *create_tray_icon() {

    tray_icon = gtk_status_icon_new();
    g_signal_connect(G_OBJECT(tray_icon), "activate", G_CALLBACK(tray_icon_on_click), NULL);
    g_signal_connect(G_OBJECT(tray_icon), "popup-menu", G_CALLBACK(tray_icon_on_menu), NULL);

    networkblank_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkblank_xpm);
    networkboth_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkboth_xpm);
    networkdead_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkdead_xpm);
    networkin_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkin_xpm);
    networkout_pixbuf=gdk_pixbuf_new_from_xpm_data((const char**)networkout_xpm);

    gtk_status_icon_set_from_pixbuf(tray_icon,networkblank_pixbuf);

    gtk_status_icon_set_tooltip(tray_icon, "No active network interfaces");
    gtk_status_icon_set_visible(tray_icon, TRUE);
    return tray_icon;
}

int main(int argc, char **argv) {

    gtk_init(&argc, &argv);
    tray_icon = create_tray_icon();
        
    gtk_timeout_add(interval, Update, NULL);

    gtk_main();
    return 0;
}
